@@ -45,6 +45,7 @@ enum wined3d_gl_extension
     /* ARB */
     ARB_BLEND_FUNC_EXTENDED,
     ARB_COLOR_BUFFER_FLOAT,
+    ARB_COPY_BUFFER,
     ARB_DEBUG_OUTPUT,
     ARB_DEPTH_BUFFER_FLOAT,
     ARB_DEPTH_TEXTURE,
@@ -91,6 +92,7 @@ enum wined3d_gl_extension
     ARB_TEXTURE_RECTANGLE,
     ARB_TEXTURE_RG,
     ARB_TEXTURE_RGB10_A2UI,
+    ARB_TEXTURE_SWIZZLE,
     ARB_TIMER_QUERY,
     ARB_UNIFORM_BUFFER_OBJECT,
     ARB_VERTEX_ARRAY_BGRA,
